#ifndef	__PY_TINKER_CONSTRUCT__
#define	__PY_TINKER_CONSTRUCT__

#include "__py_tinker_common.h"
#include "__py_tinker_func.h"

namespace py_tinker {

inline
void default_del_inst(void* ptr)
{delete ptr;}

template<typename CLS>
struct SConstruct0 : IFunction
{
	typedef CLS* (*NEW_INST)();
	typedef void (*DEL_INST)(void*);

	SConstruct0(NEW_INST new_inst, DEL_INST del_inst)
	{
		m_new_inst = new_inst ? new_inst : new_default;
		m_del_inst = del_inst ? del_inst : del_default;
	}
	virtual ~SConstruct0() {}

	PyObject* operator()(PyObject* self, PyObject* args)
	{return PyCObject_FromVoidPtr((void*)m_new_inst(), m_del_inst);}

	static CLS* new_default()
	{return new CLS;}

	static void del_default(void* ptr)
	{delete static_cast<CLS*>(ptr);}

	NEW_INST m_new_inst;
	DEL_INST m_del_inst;
};

template<typename CLS, typename ARG>
struct SConstruct1 : IFunction
{
	typedef CLS* (*NEW_INST)(ARG arg);
	typedef void (*DEL_INST)(void*);

	SConstruct1(NEW_INST new_inst, DEL_INST del_inst)
	{
		m_new_inst = new_inst ? new_inst : new_default;
		m_del_inst = del_inst ? del_inst : del_default;
	}
	virtual ~SConstruct1() {}
	PyObject* operator()(PyObject* self, PyObject* args)
	{return PyCObject_FromVoidPtr((void*)new CLS(arg_<ARG>(args, 0)), default_del_inst);}

	static CLS* new_default(ARG arg)
	{return new CLS(arg);}

	static void del_default(void* ptr)
	{delete static_cast<CLS*>(ptr);}

	NEW_INST m_new_inst;
	DEL_INST m_del_inst;
};

template<typename CLS, typename ARG, typename ARG2>
struct SConstruct2 : IFunction
{
	typedef CLS* (*NEW_INST)(ARG arg, ARG2 arg2);
	typedef void (*DEL_INST)(void*);

	SConstruct2(NEW_INST new_inst, DEL_INST del_inst)
	{
		m_new_inst = new_inst ? new_inst : new_default;
		m_del_inst = del_inst ? del_inst : del_default;
	}
	virtual ~SConstruct2() {}
	PyObject* operator()(PyObject* self, PyObject* args)
	{return PyCObject_FromVoidPtr((void*)new CLS(arg_<ARG>(args, 0), arg_<ARG2>(args, 1)), default_del_inst);}

	static CLS* new_default(ARG arg, ARG2 arg2)
	{return new CLS(arg, arg2);}

	static void del_default(void* ptr)
	{delete static_cast<CLS*>(ptr);}

	NEW_INST m_new_inst;
	DEL_INST m_del_inst;

};

template<typename CLS, typename ARG, typename ARG2, typename ARG3>
struct SConstruct3 : IFunction
{
	typedef CLS* (*NEW_INST)(ARG arg, ARG2 arg2, ARG3 arg3);
	typedef void (*DEL_INST)(void*);

	SConstruct3(NEW_INST new_inst, DEL_INST del_inst)
	{
		m_new_inst = new_inst ? new_inst : new_default;
		m_del_inst = del_inst ? del_inst : del_default;
	}

	virtual ~SConstruct3() {}
	PyObject* operator()(PyObject* self, PyObject* args)
	{return PyCObject_FromVoidPtr((void*)new CLS(arg_<ARG>(args, 0), arg_<ARG2>(args, 1), arg_<ARG3>(args, 2)), default_del_inst);}

	static CLS* new_default(ARG arg, ARG2 arg2, ARG3 arg3)
	{return new CLS(arg, arg2, arg3);}

	static void del_default(void* ptr)
	{delete static_cast<CLS*>(ptr);}

	NEW_INST m_new_inst;
	DEL_INST m_del_inst;

};

template<typename CLS, typename ARG, typename ARG2, typename ARG3, typename ARG4>
struct SConstruct4 : IFunction
{
	typedef CLS* (*NEW_INST)(ARG arg, ARG2 arg2, ARG3 arg3, ARG4 arg4);
	typedef void (*DEL_INST)(void*);

	SConstruct4(NEW_INST new_inst, DEL_INST del_inst)
	{
		m_new_inst = new_inst ? new_inst : new_default;
		m_del_inst = del_inst ? del_inst : del_default;
	}

	virtual ~SConstruct4() {}
	PyObject* operator()(PyObject* self, PyObject* args)
	{return PyCObject_FromVoidPtr((void*)new CLS(arg_<ARG>(args, 0), arg_<ARG2>(args, 1), arg_<ARG3>(args, 2), arg_<ARG4>(args, 3)), default_del_inst);}

	static CLS* new_default(ARG arg, ARG2 arg2, ARG3 arg3, ARG4 arg4)
	{return new CLS(arg, arg2, arg3, arg4);}

	static void del_default(void* ptr)
	{delete static_cast<CLS*>(ptr);}

	NEW_INST m_new_inst;
	DEL_INST m_del_inst;

}; 

} // py_tinker
#endif
