#ifndef __ETER_SYSTEM__
#define __ETER_SYSTEM__

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <eter/types.h>
#include <string>

namespace sys {

class IEventHandler
{
public:
	virtual ~IEventHandler() {}

	virtual bool OnStartup() = 0;
	virtual void OnCleanup() = 0;
	virtual void OnIdle() = 0;
	virtual void OnKeyDown(uint vkey) = 0;
	virtual void OnKeyUp(uint vkey) = 0;
	virtual void OnChangeScreenResolution(uint width, uint height) = 0;

};

class IApplication
{
public:
	static IApplication& GetInstance();

public:
	virtual LRESULT WINAPI OnDispatchMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) = 0;

	virtual bool InitConfigValueb(const char* c_name, bool value) = 0;
	virtual bool InitConfigValueu(const char* c_name, uint value) = 0;
	virtual bool Startup() = 0;
	virtual void Cleanup() = 0;

	virtual void Show() = 0;
	virtual void Hide() = 0;
	virtual void Loop() = 0;

	virtual void SetEventHandler(IEventHandler* pkEventHandler) = 0;

	virtual HWND GetMainWindowHandle() = 0;
	virtual uint GetConfigValueu(const std::string& c_rst_name) = 0;
	virtual bool GetConfigValueb(const std::string& c_rst_name) = 0;

protected:
	IApplication() {}
	virtual ~IApplication() {}
};

bool RunMain(IEventHandler* pkSysEventHandler);

} // sys

#endif