#include "StdAfx.h"
#include "d3d_object.h"

#include <eter/graphics.h>

using namespace d3d8_state_mgr;

class CVertexBuffer : public IVertexBuffer
{
public:
	CVertexBuffer()
	{
		__Initialize();
	}
	virtual ~CVertexBuffer()
	{
		Destroy();
	}
	void __Initialize()
	{
		m_pkd3dVB = NULL;
	}
	void Destroy()
	{
		if (m_pkd3dVB)
			m_pkd3dVB->Release();

		__Initialize();
	}
	bool Create(uint size, const VertexBufferSetting& c_rkSetting)
	{
		if (m_pkd3dVB)
			return false;

		IDirect3DDevice8& rkD3DDev = GetDeviceRef();
		if (FAILED(rkD3DDev.CreateVertexBuffer(
				size, 
				c_rkSetting.usage, 
				c_rkSetting.format, 
				c_rkSetting.e_pool,				
				&m_pkd3dVB)
		)) return false;
		
		return true;
	}
	bool Write(uint dstPos, uint srcSize, const void* c_pvSrcData, dword lockFlags)
	{
		if (!m_pkd3dVB)
			return false;

		IDirect3DVertexBuffer8& rkVB = *m_pkd3dVB;
		BYTE* pbDstData;
		if (FAILED(
			rkVB.Lock(dstPos, srcSize, &pbDstData, lockFlags)
		)) return false;
		
		memcpy(pbDstData, c_pvSrcData, srcSize);
		rkVB.Unlock();
		
		return true;
	}

	virtual void Apply(uint stage, uint stride)
	{
		SetStreamSource(stage, m_pkd3dVB, stride);
	}

	virtual IDirect3DVertexBuffer8* GetDirect3DVertexBufferPtr()
	{return m_pkd3dVB;}

private:
	IDirect3DVertexBuffer8* m_pkd3dVB;
};

IVertexBuffer* IVertexBuffer::NewVertexBuffer()
{
	return new CVertexBuffer;
}

