#pragma once

#include "d3d_state_mgr.h"

#include <eter/smart_pointer>

// D3D_VERTEX_BUFFER
struct VertexBufferSetting
{
	uint		usage;
	uint		format;
	D3DPOOL		e_pool;

	VertexBufferSetting() : usage(D3DUSAGE_WRITEONLY), format(D3DFVF_XYZ), e_pool(D3DPOOL_MANAGED) {}
	VertexBufferSetting(uint usage, uint format, D3DPOOL e_pool) : usage(usage), format(format), e_pool(e_pool) {}
};

class IVertexBuffer : public eter::smart_pointer_object<IVertexBuffer>
{	
public:
	static IVertexBuffer* NewVertexBuffer();

public:
	virtual ~IVertexBuffer() {}

	virtual void Destroy() {}
	virtual bool Create(uint size, const VertexBufferSetting& c_rkSetting) {return false;}
	virtual bool Write(uint dstPos, uint srcSize, const void* c_pvSrcData, DWORD lockFlag) {return false;}

	virtual void Apply(uint stage, uint stride) {}
};

typedef eter::smart_pointer<IVertexBuffer> SPVertexBuffer;
// END_OF_D3D_VERTEX_BUFFER

// D3D_INDEX_BUFFER
struct IndexBufferSetting
{
	uint		usage;
	D3DFORMAT	eFormat;
	D3DPOOL		ePool;

	IndexBufferSetting() : usage(D3DUSAGE_WRITEONLY), eFormat(D3DFMT_INDEX16), ePool(D3DPOOL_MANAGED) {}
	IndexBufferSetting(uint usage, D3DFORMAT	eFormat, D3DPOOL ePool) : usage(usage), eFormat(eFormat), ePool(ePool) {}
};

class IIndexBuffer : public eter::smart_pointer_object<IIndexBuffer>
{
public:
	static IIndexBuffer* NewIndexBuffer();

public:
	virtual ~IIndexBuffer() {}

	virtual void Destroy() {}
	virtual bool Create(uint size, const IndexBufferSetting& c_rkSetting) {return false;}
	virtual bool Write(uint dstPos, uint srcSize, const void* c_pvSrcData, DWORD lockFlag) {return false;}

	virtual void Apply(uint baseIndex) {}
};

typedef eter::smart_pointer<IIndexBuffer> SPIndexBuffer;
// END_OF_INDEX_BUFFER

