#include "StdAfx.h"
#include "d3d_state_mgr.h"
#include "d3d_vbring.h"

#include <vector>
#include <algorithm>

using namespace std;
using namespace d3d8_state_mgr;

class CVertexBufferRing : public IVertexBufferRing
{
public:
	CVertexBufferRing() : m_curIndex(0), m_bufferSize(0), m_vertexFormat(0)
	{
	}
	virtual ~CVertexBufferRing()
	{
		Destroy();
	}
	bool Create(uint vertexFormat, uint bufferSize, uint bufferCount)
	{
		assert(m_vec_pkd3dVB.empty());

		m_vertexFormat = vertexFormat;
		m_bufferSize = bufferSize;
		m_curIndex = 0;

		m_vec_pkd3dVB.reserve(bufferCount);
		
		for (uint i = 0; i < bufferCount; ++i)
		{
			IDirect3DVertexBuffer8* pkd3dNewVB = __CreateVertexBuffer();
			if (pkd3dNewVB)
				m_vec_pkd3dVB.push_back(pkd3dNewVB);
		}
		return true;
	}
	void Destroy()
	{
		for_each(m_vec_pkd3dVB.begin(), m_vec_pkd3dVB.end(), __DestroyVertexBuffer);
		m_vec_pkd3dVB.clear();
	}
	bool SetVertices(const uint c_size, const void* c_srcVertices, const uint vertexStride)
	{
		IDirect3DVertexBuffer8* pkd3dCurVB = __CopyVertices(c_size, c_srcVertices);
		if (!pkd3dCurVB)
		{
			SetStreamSource(0, NULL, 0);
			return false;
		}
			
		d3d8_state_mgr::GetDeviceRef().SetStreamSource(0, pkd3dCurVB, vertexStride);
		return true;
	}

private:
	IDirect3DVertexBuffer8* __CopyVertices(const uint c_size, const void* c_srcVertices)
	{
		if (c_size > m_bufferSize)
			return NULL;

		m_curIndex %= m_vec_pkd3dVB.size();
		IDirect3DVertexBuffer8* pkd3dCurVB = m_vec_pkd3dVB[m_curIndex++];

		BYTE* pbDstMem;
		if (FAILED(
			pkd3dCurVB->Lock(0, c_size, &pbDstMem, D3DLOCK_DISCARD)
		)) return NULL;

		memcpy(pbDstMem, c_srcVertices, c_size);

		pkd3dCurVB->Unlock();
		return pkd3dCurVB;
	}

private:
	IDirect3DVertexBuffer8* __CreateVertexBuffer()
	{
		IDirect3DDevice8* pkd3dDev = GetDevicePtr();

		IDirect3DVertexBuffer8* pkd3dNewVB = NULL;
		if (FAILED(
			pkd3dDev->CreateVertexBuffer(m_bufferSize, D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY, m_vertexFormat, D3DPOOL_SYSTEMMEM, &pkd3dNewVB)
		)) return NULL;

		return pkd3dNewVB;
	}
	static void __DestroyVertexBuffer(IDirect3DVertexBuffer8* pkd3dDelVB)
	{
		if (pkd3dDelVB)
			pkd3dDelVB->Release();
	}

private:
	vector<IDirect3DVertexBuffer8*> m_vec_pkd3dVB;

	uint m_curIndex;
	uint m_bufferSize;
	uint m_vertexFormat;
};

bool IVertexBufferRing::SetVertices(const uint c_size, const void* c_srcVertices, const uint vertexStride)
{
	SetStreamSource(0, NULL, 0);
	return false;
}

IVertexBufferRing* IVertexBufferRing::NewVertexBufferRing(uint vertexFormat, uint bufferSize, uint bufferCount)
{
	CVertexBufferRing* vpkNewVBRing = new CVertexBufferRing;
	if (!vpkNewVBRing->Create(vertexFormat, bufferSize, bufferCount))
	{
		delete vpkNewVBRing;

		return IVertexBufferRing::None();
	}

	return vpkNewVBRing;
}

