#include "StdAfx.h"
#include "d3d_state_mgr.h"

#include <eter/graphics.h>
#include <assert.h>
#include <stdio.h>

#include <string>

namespace grp {

using namespace std;

class CDevice : public IDevice
{
public:
	enum
	{
		D3DERR_UNKNOWN,
		D3DERR_ALREADYEXIST,
		D3DERR_NOTEXISTHWND,		
	};

public:
	CDevice()
	{
		m_pkd3d		= NULL;
		m_pkd3dDev	= NULL;

		m_hWndMain	= NULL;
		
		m_d3dBehavior = 0;
		m_id3dModeInfo	= 0;
		m_id3dDevInfo	= 0;
		m_id3dAdapterInfo = 0;

		memset(&m_d3dPresentParameter, 0, sizeof(m_d3dPresentParameter));
		
	}
	virtual ~CDevice()
	{
		assert(NULL == m_pkd3dDev);
		assert(NULL == m_pkd3d);
	}

	virtual int  BackupStateBlock()
	{
		return d3d8_state_mgr::BackupStateBlock();
	}
	virtual void RestoreStateBlock(int handle)
	{
		d3d8_state_mgr::RestoreStateBlock(handle);
	}

	virtual void Clear()
	{
		if (m_pkd3dDev)
			m_pkd3dDev->Clear(0, NULL, m_clearSetting.flags, m_clearSetting.color, m_clearSetting.depth, m_clearSetting.stencil);
	}
	virtual bool BeginScene()
	{
		if (m_pkd3dDev)
			if (FAILED(
				m_pkd3dDev->BeginScene()
			)) return false;
			
		return true;
	}
	virtual void EndScene()
	{
		if (m_pkd3dDev)
			m_pkd3dDev->EndScene();
	}
	virtual void Present()
	{
		if (m_pkd3dDev)
			m_pkd3dDev->Present(NULL, NULL, NULL, NULL);
	}

	void InitWindowHandle(HWND hWnd)
	{
		m_hWndMain = hWnd;
	}
	bool InitConfigValueb(const char* c_name, bool value)
	{
		if (strcmp(c_name, "isWindowed") == 0)
			m_config.isWindowed = value ? true : false;
		else
		{
			__SysErr_UnknownProperty("b", c_name);
			return false;
		}
		return true;
	}
	bool InitConfigValueu(const char* c_name, uint value)
	{
		if (strcmp(c_name, "hres") == 0)
			m_config.hres = value;
		else if (strcmp(c_name, "vres") == 0)
			m_config.vres = value;
		else if (strcmp(c_name, "backBufferCount") == 0)
			m_config.backBufferCount = value;
		else
		{
			__SysErr_UnknownProperty("u", c_name);
			return false;
		}
		return true;
	}
	bool Startup()
	{
		if (m_pkd3d || m_pkd3dDev)
		{
			__SetErrType(enum2str(D3DERR_ALREADYEXIST));
			return false;
		}

		if (!m_hWndMain)
		{
			__SetErrType(enum2str(D3DERR_NOTEXISTHWND));
			return false;
		}

		m_pkd3d = Direct3DCreate8(D3D_SDK_VERSION);
		if (!m_pkd3d)
		{
			__SetErrType(enum2str(D3DERR_INITFAILED));
			return false;
		}		

		if (!__DetectDisplayMode())
		{
			return false;
		}
				
		D3DPRESENT_PARAMETERS& rkD3DPresentParameter=m_d3dPresentParameter;
		memset(&rkD3DPresentParameter, 0, sizeof(rkD3DPresentParameter));
		rkD3DPresentParameter.Windowed			= m_config.isWindowed;
		rkD3DPresentParameter.SwapEffect		= D3DSWAPEFFECT_DISCARD;
		rkD3DPresentParameter.BackBufferWidth	= m_config.hres;
		rkD3DPresentParameter.BackBufferHeight	= m_config.vres;
		rkD3DPresentParameter.hDeviceWindow		= m_hWndMain;
		rkD3DPresentParameter.BackBufferCount	= m_config.backBufferCount;
		
		if (rkD3DPresentParameter.Windowed)
		{
			 rkD3DPresentParameter.BackBufferFormat = __GetDesktopBackBufferFormat();
		}
		else
		{
			rkD3DPresentParameter.BackBufferFormat				= __GetFullScreenBackBufferFormat();
			rkD3DPresentParameter.FullScreen_RefreshRateInHz	= 0;
		}
		
		rkD3DPresentParameter.AutoDepthStencilFormat = __GetDepthStencilFormat();
		rkD3DPresentParameter.EnableAutoDepthStencil = TRUE;	
		
		HRESULT hr;
		hr = m_pkd3d->CreateDevice(
			m_id3dAdapterInfo,
			D3DDEVTYPE_HAL,
			m_hWndMain,
			__GetBehaviorMode(),
			&rkD3DPresentParameter,
			&m_pkd3dDev);
		
		if (FAILED(hr))
		{
			switch (hr)
			{
			case D3DERR_INVALIDCALL:
				__SetErrType(enum2str(D3DERR_INVALIDCALL));				
				break;
			case D3DERR_NOTAVAILABLE:
				__SetErrType(enum2str(D3DERR_NOTAVAILABLE));
				break;
			case D3DERR_OUTOFVIDEOMEMORY:
				__SetErrType(enum2str(D3DERR_OUTOFVIDEOMEMORY));
				break;
			default:
				__SetErrType(enum2str(D3DERR_UNKNOWN));
				break;
			}
			return false;
		}

		d3d8_state_mgr::Startup(m_pkd3dDev);
		d3d8_state_mgr::SetDefaultState();
		
		return true;
	}
	void Cleanup()
	{
		d3d8_state_mgr::Cleanup();

		if (m_pkd3dDev)
		{
			m_pkd3dDev->Release();
			m_pkd3dDev = NULL;
		}
		
		if (m_pkd3d)
		{
			m_pkd3d->Release();
			m_pkd3d = NULL;
		}
	}	
	bool ChangeBackBufferSize(uint hres, uint vres)
	{
		if (!m_pkd3dDev)
			return false;
		
		IDirect3DDevice8& rkD3DDev = *m_pkd3dDev;
		
		D3DPRESENT_PARAMETERS& rkD3DPP = m_d3dPresentParameter;
		if (!rkD3DPP.Windowed)
			return false;
		
		{
			if (rkD3DPP.BackBufferWidth!=hres|| rkD3DPP.BackBufferHeight!=vres)
			{
				rkD3DPP.BackBufferWidth=hres;
				rkD3DPP.BackBufferHeight=vres;
				
				HRESULT hr=rkD3DDev.Reset(&rkD3DPP);
				if (FAILED(hr))
					return false;
				
				d3d8_state_mgr::SetDefaultState();
			}
		}
		return true;
	}

	bool RestoreDevice()
	{
		assert(NULL != m_pkd3d);
		assert(NULL != m_pkd3dDev);

		UINT id3dAdapterInfo = m_id3dAdapterInfo;
		IDirect3D8& rkD3D = *m_pkd3d;
		IDirect3DDevice8& rkD3DDev = *m_pkd3dDev;
		D3DPRESENT_PARAMETERS& rkD3DPP = m_d3dPresentParameter;
		
		HRESULT hrTestCooperativeLevel = rkD3DDev.TestCooperativeLevel();

		if (FAILED(hrTestCooperativeLevel))
		{		
			if (D3DERR_DEVICELOST == hrTestCooperativeLevel)
			{
				__SysErr("CDevice::Restore - D3DERR_DEVICELOST");
				return false;		
			}

			if (D3DERR_DEVICENOTRESET == hrTestCooperativeLevel)
			{
				rkD3DPP.BackBufferFormat = __GetDesktopBackBufferFormat();
				
				HRESULT hrReset = rkD3DDev.Reset(&rkD3DPP);
				if (FAILED(hrReset))
				{
					__SysErr("CDevice::Restore - IDirect3DDevice8::Reset FAILURE");
					return false;
				}

				d3d8_state_mgr::SetDefaultState();
			}        
		}

		return true;

	}

	virtual IDirect3DDevice8* GetDirect3DDevicePtr()
	{
		assert(NULL != m_pkd3dDev && "GetDirect3DDevicePtr");
		return m_pkd3dDev;
	}
private:
	bool __DetectDisplayMode()
	{
	    if (FAILED( 
			m_pkd3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &m_d3dDisplayMode) 
		)) return false;

		return true;
	}
	void __SetErrType(const std::string& c_rst_errType)
	{
		m_st_errType = c_rst_errType;
		__SysErr(c_rst_errType.c_str());
	}
	void __SysErr_UnknownProperty(const char* tail, const string& c_rst_name)
	{
		char buf[256];
		_snprintf(buf, sizeof(buf)-1, "CDevice::InitConfigValue%s - UNKNOWN PROPERTY[%s]", tail, c_rst_name.c_str());
		__SysErr(buf);
	}
	void __SysErr(const char* c_msg)
	{
		fputs(c_msg, stderr);
	}
	dword __GetBehaviorMode()
	{
		// TODO: ÿ ´ ؽμ 带 Ѵ.
		return D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	}
	D3DFORMAT __GetDepthStencilFormat()
	{
		// TODO: ̽ٽ   Ѵ.
		return D3DFMT_D16;
	}
	D3DFORMAT __GetDesktopBackBufferFormat()
	{
		return m_d3dDisplayMode.Format;
	}
	D3DFORMAT __GetFullScreenBackBufferFormat()
	{
		// TODO: Ǯũ    Ѵ. 
		return D3DFMT_A8R8G8B8;
	}

private:
	IDirect3D8*				m_pkd3d;
	IDirect3DDevice8*		m_pkd3dDev;
	D3DPRESENT_PARAMETERS	m_d3dPresentParameter;
	D3DDISPLAYMODE			m_d3dDisplayMode;

	HWND	m_hWndMain;

	uint	m_d3dClearFlags;
	uint	m_d3dBehavior;
	uint	m_id3dModeInfo;
	uint	m_id3dDevInfo;
	uint	m_id3dAdapterInfo;

	string	m_st_errType;

	struct Config
	{
		uint hres;
		uint vres;
		uint backBufferCount;
		bool isWindowed;

		Config() : hres(640), vres(480), backBufferCount(3), isWindowed(true) {}
	} m_config;

	struct ClearSetting
	{
		uint flags;
		uint color;
		float depth;
		float stencil;

		ClearSetting() : flags(D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER), color(0xff777777), depth(1.0f), stencil(0.0f) {}

	} m_clearSetting;
};


IDevice& IDevice::GetInstance()
{
	static CDevice s_kGraphicDevice;
	return s_kGraphicDevice;
}

} // grp