#include "StdAfx.h"

#include <eter/graphics.h>
#include <eter/system.h>

namespace sys {

static void __Cleanup()
{
	sys::IApplication& rkSysApp = sys::IApplication::GetInstance();
	grp::IDevice& rkGrpDev = grp::IDevice::GetInstance();

	rkGrpDev.Cleanup();
	rkSysApp.Cleanup();
}

static bool __Startup()
{
	sys::IApplication& rkSysApp = sys::IApplication::GetInstance();
	grp::IDevice& rkGrpDev = grp::IDevice::GetInstance();

	if (!rkSysApp.Startup())
		return false;
	
	HWND hWndMain = rkSysApp.GetMainWindowHandle();
	uint hres = rkSysApp.GetConfigValueu("hres");
	uint vres = rkSysApp.GetConfigValueu("vres");
	rkGrpDev.InitWindowHandle(hWndMain);
	rkGrpDev.InitConfigValueu("hres", hres);
	rkGrpDev.InitConfigValueu("vres", vres);
	if (!rkGrpDev.Startup())
		return false;

	return true;
}

static void __Loop()
{
	sys::IApplication& rkSysApp = sys::IApplication::GetInstance();
	
	rkSysApp.Show();
	rkSysApp.Loop();
	rkSysApp.Hide();
}

bool RunMain(sys::IEventHandler* pkSysEventHandler)
{
	sys::IApplication& rkSysApp = sys::IApplication::GetInstance();
	
	rkSysApp.SetEventHandler(pkSysEventHandler);

	bool ret = false;
	if (__Startup())
	{
		if (pkSysEventHandler)
			pkSysEventHandler->OnStartup();

		__Loop();

		ret = true;
	}

	if (pkSysEventHandler)
		pkSysEventHandler->OnCleanup();

	__Cleanup();

	return ret;
}
} // sys