#include "StdAfx.h"

#include <eter/system.h>
#include <windowsx.h>

namespace sys {

using namespace std;

LRESULT WINAPI __OnDispatchMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

static const char* ETER_APPLICATION_CLASSNAME = "eter_application";

static const WNDCLASSEX ETER_APPLICATION_WNDCLASSEX = 
{ 
	sizeof(WNDCLASSEX), CS_CLASSDC, __OnDispatchMessage, 
	0L, 0L, GetModuleHandle(NULL), NULL, NULL, 
	(HBRUSH)GetStockObject(BLACK_BRUSH), NULL,
	ETER_APPLICATION_CLASSNAME, NULL 
};



class CApplication : public IApplication
{
public:
	CApplication()
	{
	}
	virtual ~CApplication()
	{
	}
	bool InitConfigValueb(const char* c_name, bool value)
	{
		if (strcmp(c_name, "isEnableIdle") == 0)
			m_config.isEnableIdle = value ? true : false;
		else
		{
			__SysErr_UnknownProperty("b", c_name);
			return false;
		}
		return true;
	}
	bool InitConfigValueu(const char* c_name, uint value)
	{
		if (strcmp(c_name, "hres") == 0)
			m_config.hres = value;
		else if (strcmp(c_name, "vres") == 0)
			m_config.vres = value;
		else
		{
			__SysErr_UnknownProperty("u", c_name);
			return false;
		}
		return true;
	}
	LRESULT WINAPI OnDispatchMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
	{
		if (m_pkEventHandler)
		{
			switch (msg)
			{
			case WM_KEYDOWN:
				m_pkEventHandler->OnKeyDown(LOWORD(wParam));
				break;
				
			case WM_KEYUP:
				m_pkEventHandler->OnKeyUp(LOWORD(wParam));
				break;
				
				// WINDOWS 98  Ǯȭȯ WM_SIZE ڰ  ȭ ػ󵵷
				//  Ǳ⶧, GetClientRect   ũ⸦  Ѵ.
			case WM_SIZE: 
				{
					switch (wParam)
					{
					case SIZE_RESTORED:
					case SIZE_MAXIMIZED:
						{
							RECT rcClient; 
							GetClientRect(hWnd, &rcClient);
							m_pkEventHandler->OnChangeScreenResolution(rcClient.right - rcClient.left, rcClient.bottom - rcClient.left);						
						}
						break;
					}			
				}
				break;			
			case WM_EXITSIZEMOVE:
				{
					RECT rcClient; 
					GetClientRect(hWnd, &rcClient);
					m_pkEventHandler->OnChangeScreenResolution(rcClient.right - rcClient.left, rcClient.bottom - rcClient.left);						
				}
				break;		
				
			}
		}

		switch (msg)
		{
		case WM_PAINT:
            ValidateRect(hWnd, NULL);
            return 0;
			
		case WM_DESTROY:
			PostQuitMessage( 0 );
			return 0;
		}
		
		return DefWindowProc( hWnd, msg, wParam, lParam );		
	}
	bool Startup()
	{
		RegisterClassEx(&ETER_APPLICATION_WNDCLASSEX);

		m_hWndMain = CreateWindow( 
			ETER_APPLICATION_CLASSNAME, m_config.st_title.c_str(), 
			WS_OVERLAPPEDWINDOW, 0, 0, 0, 0,
			GetDesktopWindow(), NULL, ETER_APPLICATION_WNDCLASSEX.hInstance, NULL );
		
		__AdjustWindowSize(m_hWndMain, m_config.hres, m_config.vres);
		
		__CentreWindow(m_hWndMain);
		
		return true;	
	}
	void Cleanup()
	{
		UnregisterClass(ETER_APPLICATION_CLASSNAME, ETER_APPLICATION_WNDCLASSEX.hInstance);
	}
	void Show()
	{
		ShowWindow(m_hWndMain, SW_SHOWDEFAULT);
		UpdateWindow(m_hWndMain);
	}
	void Hide()
	{
		ShowWindow(m_hWndMain, SW_HIDE);
	}
	void Loop()
	{
		MSG msg; 
		while(1)
		{
			if (m_config.isEnableIdle)
			{
				if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
				{
					if (!GetMessage(&msg, NULL, 0, 0 ))
						break;
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
				else
				{
					if (m_pkEventHandler)
						m_pkEventHandler->OnIdle();
				}
			}
			else
			{
				if (!GetMessage(&msg, NULL, 0, 0 ))
					break;
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}

	}
	virtual void SetEventHandler(IEventHandler* pkEventHandler)
	{
		m_pkEventHandler = pkEventHandler;
	}
	virtual HWND GetMainWindowHandle()
	{
		return m_hWndMain;
	}
	virtual uint GetConfigValueu(const std::string& c_rst_name)
	{
		if (c_rst_name == "hres")
			return m_config.hres;
		else if (c_rst_name == "vres")
			return m_config.vres;
		else
			__SysErr_UnknownProperty("u", c_rst_name);
		return 0;
	}
	virtual bool GetConfigValueb(const std::string& c_rst_name)
	{
		__SysErr_UnknownProperty("u", c_rst_name);
		return false;
	}
	
private:
	void __CentreWindow(HWND hWnd)
	{
		RECT rc;
		GetClientRect(hWnd, &rc);
		
		int windowWidth = rc.right - rc.left;
		int windowHeight = rc.bottom - rc.top;
		
		int screenWidth = GetSystemMetrics(SM_CXSCREEN);
		int screenHeight = GetSystemMetrics(SM_CYSCREEN);
		
		int xCenter = (screenWidth-windowWidth)/2;
		int yCenter = (screenHeight-windowHeight)/2;
		
		SetWindowPos(hWnd, NULL, xCenter, yCenter, 0, 0, SWP_NOZORDER|SWP_NOSIZE);	
	}
	void __AdjustWindowSize(HWND hWnd, uint width, uint height)
	{
		RECT kRect;
		SetRect(&kRect, 0, 0, width, height);
		
		AdjustWindowRectEx(&kRect,
			GetWindowStyle(hWnd),     
			GetMenu(hWnd) != NULL,    
			GetWindowExStyle(hWnd ) ); 
		
		MoveWindow(hWnd, 0, 0, kRect.right - kRect.left, kRect.bottom - kRect.top, FALSE);
	}
	void __SysErr_UnknownProperty(const char* tail, const string& c_rst_name)
	{
		char buf[256];
		_snprintf(buf, sizeof(buf)-1, "grp_CDevice::InitConfigValue%s - UNKNOWN PROPERTY[%s]", tail, c_rst_name.c_str());
		__SysErr(buf);
	}
	void __SysErr(const char* c_msg)
	{
		fputs(c_msg, stderr);
	}

private:
	HWND m_hWndMain;

	IEventHandler* m_pkEventHandler;

	struct Config
	{
		string	st_title;
		uint	hres;
		uint	vres;
		bool	isEnableIdle;

		Config() : st_title("eter.sys.application"), hres(640), vres(480), isEnableIdle(true) {}
	} m_config;
};

IApplication& IApplication::GetInstance()
{
	static CApplication s_kSysApp;
	return s_kSysApp;
}

LRESULT WINAPI __OnDispatchMessage(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	IApplication& rkSysApp = IApplication::GetInstance();
	return rkSysApp.OnDispatchMessage(hWnd, msg, wParam, lParam);
}

} // sys 