// test.cpp : Defines the entry point for the console application.
#include "stdafx.h"
#include "graphics.h"
#include "system.h"
#include "main.h"

#include "il/il.h"

#include "test_image.h"

#include <string>
#include <vector>
#include <map>

#include "method"
#include "string_utils"
#include "smart_pointer"

using namespace py_tinker;
using namespace std;


extern void initeter();
extern void inittest_image();

class eter_CPythonEventHandler : public sys::IEventHandler
{
public:
	static eter_CPythonEventHandler& GetInstance()
	{
		assert(NULL != ms_pkPyEventHandler);
		return *ms_pkPyEventHandler;
	}

private:
	static eter_CPythonEventHandler* ms_pkPyEventHandler;
	

public:
	eter_CPythonEventHandler()
	{
		assert(NULL == ms_pkPyEventHandler);
		ms_pkPyEventHandler = this;
	}
	void SetScriptFileName(const string& c_rst_scriptFileName)
	{
		m_st_scriptFileName = c_rst_scriptFileName;
	}

	void ShowEventList()
	{
		using namespace eter;

		string st_eventNames;
		string_join(", ", m_vec_st_eventName, &st_eventNames);

		fprintf(stdout, "[%s]", st_eventNames.c_str());
	}
	bool BindEvent(PyObject* eventName, PyObject* eventFunc)
	{
		if (!PyCallable_Check(eventFunc))
		{
			throw PyTinkerException(PyExc_RuntimeError, "eventFunc is NOT CALLABLE");
			return false;
		}

		long hash = PyObject_Hash(eventName);

		map<long, object*>::iterator f = m_map_hash_eventFunc.find(hash);
		if (m_map_hash_eventFunc.end() == f)
		{
			char msg[256];
			snprintf(msg, sizeof(msg)-1, "eventName[%s] is NOT EXIST", PyString_AsString(eventName));
			throw PyTinkerException(PyExc_RuntimeError, msg);
			return false;
		}

		*f->second = eventFunc;
		
		return true;
	}

	bool OnStartup()
	{
		test_image::Startup();

		Py_Initialize();
		Py_SetProgramName("eter.game");

		__InitEventDict();

		initeter();
		inittest_image();
		
		return PyEmbed_RunMainFile(m_st_scriptFileName.c_str());
	}
	void OnCleanup()
	{
		Py_Finalize();

		test_image::Cleanup();
	}
	void OnIdle()
	{
		grp::IDevice& rkGrpDev = grp::IDevice::GetInstance();
		__OnRender(rkGrpDev);
		Sleep(100);
	}
	void OnKeyUp(uint vkey) 
	{m_onKeyUp(vkey);}
	void OnKeyDown(uint vkey) 
	{m_onKeyDown(vkey);}

	void OnChangeScreenResolution(HWND hWnd) 
	{
		RECT rcClient; 
		GetClientRect(hWnd, &rcClient);
		
		UINT hres=rcClient.right-rcClient.left; 
		UINT vres=rcClient.bottom-rcClient.left; 

		grp::IDevice& rkGrpDev = grp::IDevice::GetInstance();
		rkGrpDev.ChangeBackBufferSize(hres, vres);
	}

private:
	void __OnRender(grp::IDevice& rkGrpDev)
	{
		rkGrpDev.Clear();
		if (rkGrpDev.BeginScene())
		{
			m_onRender();
			rkGrpDev.EndScene();
			rkGrpDev.Present();
		}
	}
	void __InitEventDict()
	{
		__InitEvent("ON_IDLE", &m_onIdle);
		__InitEvent("ON_RENDER", &m_onRender);
		__InitEvent("ON_KEY_DOWN", &m_onKeyDown);
		__InitEvent("ON_KEY_UP", &m_onKeyUp);

		
	}
	void __InitEvent(const char* eventName, object* pkEventFunc)
	{
		object kEventName = PyString_FromString(eventName);
		long hash = PyObject_Hash(kEventName);
		m_map_hash_eventFunc.insert(map<long, object*>::value_type(hash, pkEventFunc));	

		m_vec_st_eventName.push_back(eventName);
	}
	
private:
	string m_st_scriptFileName;

	map<long, object*>	m_map_hash_eventFunc;
	vector<string>		m_vec_st_eventName;

	object m_onIdle;
	object m_onRender;
	object m_onKeyDown;
	object m_onKeyUp;
};

eter_CPythonEventHandler* eter_CPythonEventHandler::ms_pkPyEventHandler = NULL;

void eter_bind_event(PyObject* eventName, PyObject* eventFunc)
{
	eter_CPythonEventHandler::GetInstance().BindEvent(eventName, eventFunc);
}

static eter_CPythonEventHandler* new_sys_event_handler()
{
	return &eter_CPythonEventHandler::GetInstance();
}

static void del_sys_event_handler(void* sys_event_handler)
{}

class_<eter_CPythonEventHandler> build_sys_event_handler()
{
	return class_<eter_CPythonEventHandler>("EventHandler", construct0<eter_CPythonEventHandler>(new_sys_event_handler, del_sys_event_handler))
		.deft("BindEvent",		&eter_CPythonEventHandler::BindEvent)
		.defv("ShowEventList",	&eter_CPythonEventHandler::ShowEventList)
		;
}

module_ buildsys()
{
	return module_("sys")
		.def(build_sys_event_handler())
		;
}

void inittest_image()
{
	module_("test_image")
		.defv("RenderImage", &test_image::RenderImage)
		;
}

void initeter()
{
	module_("eter")
		.def(buildsys())
		;
}


int main(int argc, char* argv[])
{
	if (argc<2)
	{
		puts("dress_keeper [python_script]");
		return -1;
	}

	ilInit();
	eter_CPythonEventHandler kPyEventHandler;
	kPyEventHandler.SetScriptFileName(argv[1]);
	RunMain(&kPyEventHandler);
	return 0;
}
